@@ -361,6 +361,10 @@
 		;;((and (simple-format #t "chs=>~S\n" (read-chseq ch)) #f))
 		((read-chseq ch) => identity)
 		((assq-ref chrtab ch) => (lambda (t) (cons t (string ch))))
+	        ((eqv? ch #\\)
+		 (let ((ch (read-char)))
+		   (cond ((eqv? #\newline ch) (iter (read-char)))
+			 (else (unread-char ch) (cons #\\ "\\")))))
 		(else (cons ch (string ch))))))
 	    )
 
