@@ -1,6 +1,6 @@
 ;;; lang/c99/mach.scm
 ;;;
-;;; Copyright (C) 2015 Matthew R. Wette
+;;; Copyright (C) 2015,2016 Matthew R. Wette
 ;;;
 ;;; This program is free software: you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by 
@@ -18,7 +18,7 @@
 ;; C parser generator: based on ISO-C99; with comments and CPP statements
 
 (define-module (nyacc lang c99 mach)
-  #:export (clang-spec clang-mach dev-parse-c)
+  #:export (c99-spec c99-mach dev-parse-c gen-c99-files gen-c99x-files)
   #:use-module (nyacc lang c99 cpp)
   #:use-module (nyacc lalr)
   #:use-module (nyacc parse)
@@ -29,18 +29,18 @@
   #:use-module ((sxml xpath) #:select (sxpath))
   )
 
-;; Objective is to generate a sxml tree.
-;; Written to the grammar specified for ISO-C99, 
-;;   http://slps.github.io/zoo/c/iso-9899-tc3.html
-;; but modified to handle comments and CPP statements.
-(define clang-spec
+;; @item c99-spec
+;; This variable is the specification a-list for the hacked ISO C99 language.
+;; Run this through @code{make-lalr-machine} to get an a-list for the
+;; automaton.  The grammar is modified to parse CPP statements and comments.
+;; The output of the end parser will be a SXML tree (w/o the @code{*TOP*} node.
+(define c99-spec
   (lalr-spec
    (notice lang-crn-lic)
-   (prec< "then" "else")	    ; then/else SR-shift resolution
-   (prec< "imp"			    ; implied type SR-shift resolution
+   (prec< "then" "else")	    ; "then/else" SR-conflict resolution
+   (prec< "imp"			    ; "implied type" SR-conflict resolution
 	  "char" "short" "int" "long"
 	  "float" "double" "_Complex")
-   ;;(expect 25)			; 25 SR-conf fixed with above prec
    (start translation-unit-proxy)
    (grammar
 
@@ -73,18 +73,14 @@
     (argument-expression-list
      (assignment-expression ($$ (make-tl 'expr-list $1)))
      (argument-expression-list "," assignment-expression ($$ (tl-append $1 $3)))
-     ;; The following is a kludge to deal with using typenames in CPP defines
-     ;; when we don't expand the defines.  e.g., GEN_OFFSET(foo_t, x.y.z)
-     ;;   (typedef-name ($$ (make-tl 'expr-list $1)))
-     ;;   (argument-expression-list "," typedef-name ($$ (tl-append $1 $3)))
-     ;; Changed after V0.67.0 to:
+     ;; The following is a modification to deal with using abstract declarations
+     ;; as arguments to CPP macros (e.g., see offsetof in <stddef.h>).
      (arg-expr-hack ($$ (make-tl 'expr-list $1)))
      (argument-expression-list "," arg-expr-hack ($$ (tl-append $1 $3)))
      )
     (arg-expr-hack
-     (declaration-specifiers abstract-declarator ($$ `(decl ,(tl->list $1 $2))))
-     (declaration-specifiers ($$ `(decl ,(tl->list $1))))
-     )
+     (declaration-specifiers abstract-declarator ($$ `(decl ,(tl->list $1) $2)))
+     (declaration-specifiers ($$ `(decl ,(tl->list $1)))))
 
     (unary-expression
      (postfix-expression)		; S 6.5.3
@@ -625,7 +621,7 @@
      (lone-comment)
      (cpp-statement)
      ;; The following is a kludge to deal with @code{extern "C" @{}.
-     ("extern" '$string "{" translation-unit "}" ($$ (tl->list $4)))
+     ("extern" $string "{" translation-unit "}" ($$ (tl->list $4)))
      )
     
     (function-definition
@@ -648,44 +644,42 @@
 
     ;; non-terminal leaves
     (identifier
-     ('$ident ($$ `(ident ,$1)))
+     ($ident ($$ `(ident ,$1)))
      ('cpp-ident ($$ `(ident ,$1))))
     (constant
-     ('$fixed ($$ `(fixed ,$1)))	; integer-constant
-     ('$float ($$ `(float ,$1)))	; floating-constant
-     ('$chlit ($$ `(char ,$1))))	; char-constant
+     ($fixed ($$ `(fixed ,$1)))		; integer-constant
+     ($float ($$ `(float ,$1)))		; floating-constant
+     ($chlit ($$ `(char ,$1))))		; char-constant
     (string-literal
-     ('$string ($$ (make-tl 'string $1))) ; string-constant
-     (string-literal '$string ($$ (tl-append $1 $2))))
-    (code-comment ('$code-comm ($$ `(comment ,$1))))
-    (lone-comment ('$lone-comm ($$ `(comment ,$1))))
+     ($string ($$ (make-tl 'string $1))) ; string-constant
+     (string-literal $string ($$ (tl-append $1 $2))))
+    (code-comment ($code-comm ($$ `(comment ,$1))))
+    (lone-comment ($lone-comm ($$ `(comment ,$1))))
     (cpp-statement ('cpp-stmt ($$ `(cpp-stmt ,$1))))
 
     )))
 
 
-(define clang-mach
+(define c99-mach
   (compact-machine
-   ;;(identity
    (hashify-machine
-    ;;(identity
-    (make-lalr-machine clang-spec))))
+    (make-lalr-machine c99-spec))))
 
 ;;; =====================================
 
 ;; The following are needed by the code in pbody.scm.
-(define len-v (assq-ref clang-mach 'len-v))
-(define pat-v (assq-ref clang-mach 'pat-v))
-(define rto-v (assq-ref clang-mach 'rto-v))
-(define mtab (assq-ref clang-mach 'mtab))
+(define len-v (assq-ref c99-mach 'len-v))
+(define pat-v (assq-ref c99-mach 'pat-v))
+(define rto-v (assq-ref c99-mach 'rto-v))
+(define mtab (assq-ref c99-mach 'mtab))
 (define act-v (vector-map
 	       (lambda (ix f) (eval f (current-module)))
 	       (vector-map (lambda (ix actn) (wrap-action actn))
-			   (assq-ref clang-mach 'act-v))))
+			   (assq-ref c99-mach 'act-v))))
 
-(include-from-path "nyacc/lang/c99/pbody.scm")
+(include-from-path "nyacc/lang/c99/body.scm")
 
-(define raw-parser (make-lalr-parser clang-mach))
+(define raw-parser (make-lalr-parser c99-mach))
 
 (define (run-parse)
   (let ((info (fluid-ref *info*)))
@@ -706,13 +700,51 @@
 
 ;;; =====================================
 
-#;(define (gen-c99-files . rest)
-  (define (module-path path) path)
-  (write-lalr-actions clang-mach "c99act.scm.new")
-  (write-lalr-tables clang-mach "c99tab.scm.new")
-  (when (or (move-if-changed "c99act.scm.new" (module-path "c99act.scm"))
-	    (move-if-changed "c99tab.scm.new" (module-path "c99tab.scm")))
-    (system (string-append "touch " (module-path "parser.scm")))
-    (compile-file (module-path "parser.scm"))))
+;; @item gen-c99x-files [dir] => #t
+;; Update or generate the files @quot{c99xact.scm} and @quot{c99xtab.scm}.
+;; These are the tables and actions for the C99 parser.
+;; If there are no changes to existing files, no update occurs.
+(define (gen-c99-files . rest)
+  (define (lang-dir path)
+    (if (pair? rest) (string-append (car rest) "/" path) path))
+  (define (xtra-dir path)
+    (lang-dir (string-append "mach.d/" path)))
+
+  (write-lalr-actions c99-mach (xtra-dir "c99act.scm.new"))
+  (write-lalr-tables c99-mach (xtra-dir "c99tab.scm.new"))
+  (let ((a (move-if-changed (xtra-dir "c99act.scm.new")
+			    (xtra-dir "c99act.scm")))
+	(b (move-if-changed (xtra-dir "c99tab.scm.new")
+			    (xtra-dir "c99tab.scm"))))
+    (when (or a b) 
+      (system (string-append "touch " (lang-dir "parser.scm")))
+      #;(compile-file (lang-dir "parser.scm"))
+      )))
+
+;; @item gen-c99x-files [dir] => #t
+;; Update or generate the files @quot{c99xact.scm} and @quot{c99xtab.scm}.
+;; These are the tables and actions for the C99 expression parser.
+;; If there are no changes to existing files, no update occurs.
+(define (gen-c99x-files . rest)
+  (define (lang-dir path)
+    (if (pair? rest) (string-append (car rest) "/" path) path))
+  (define (xtra-dir path)
+    (lang-dir (string-append "mach.d/" path)))
+
+  (let* ((cexpr-spec (restart-spec c99-mach 'expression))
+	 (cexpr-mach (compact-machine
+		      (hashify-machine
+		       (make-lalr-machine cexpr-spec)))))
+    (write-lalr-actions cexpr-mach (xtra-dir "c99xact.scm.new"))
+    (write-lalr-tables cexpr-mach (xtra-dir "c99xtab.scm.new")))
+    
+  (let ((a (move-if-changed (xtra-dir "c99xact.scm.new")
+			    (xtra-dir "c99xact.scm")))
+	(b (move-if-changed (xtra-dir "c99xtab.scm.new")
+			    (xtra-dir "c99xtab.scm"))))
+    (when (or a b) 
+      (system (string-append "touch " (lang-dir "parser.scm")))
+      #;(compile-file (lang-dir "xparser.scm"))
+      )))
 
 ;; --- last line ---
