@@ -68,7 +68,7 @@
 ;; @item declr->ident declr => (ident "name")
 ;; just match the declarator
 ;; (init-declr <declr> [<initzer>])
-;; See also: declr->id-name in pbody.scm.
+;; See also: declr->id-name in body.scm.
 (define (declr->ident declr)
   (sxml-match declr
     ((init-declr ,declr . ,rest)
@@ -458,12 +458,7 @@
 (define (udecl->mspec decl . rest)
 
   (define (cnvt-array-size size-spec)
-    ;; This needs to be completed in order to be able to pretty-print
-    ;; constant expressions.
-    (sxml-match size-spec
-      ((p-expr (fixed ,size)) size)
-      ((p-expr (ident ,size)) size)
-      (,otherwise size-spec)))
+    (with-output-to-string (lambda () (pretty-print-c99 size-spec))))
 
   (define (unwrap-specl specl)
     (let ((tspec (cadadr specl)))
