@@ -394,7 +394,7 @@
 		(if (and (not eat-newline) (eq? #\newline (car sl)))
 		    (unread-char #\newline))
 		;;(cons '$comment (list->string (reverse cl))))
-		(if (eqv? (car cl) #\cr) ;; remove trailing \r 
+		(if (and (pair? cl) (eqv? (car cl) #\cr)) ;; rem trailing \r 
 		    (cons '$comment (list->string (reverse (cdr cl))))
 		    (cons '$comment (list->string (reverse cl)))))
 	       ((null? il) (find-end cl sl (cons (read-char) il) ps px))
